module Gh
  module Trending
    module Parsers
      # This class represents the arguments that both the trending
      # repositories and languages repositories accept.
      #
      # If these pages accept new arguments, just add a new method here
      # like:
      #   def new_arg
      #     args[:arg_name]
      #   end
      #
      # Also, complete the no_args? method.
      class ParserArguments
        attr_reader :args
        UnknownTimePeriod = Class.new(StandardError)

        TIME_PERIOD_ARG = :time_period
        LANGUAGE_ARG    = :language

        KNOWN_TIME_PERIODS = %i[daily weekly monthly].freeze
        DEFAULT_TIME_PERIOD = KNOWN_TIME_PERIODS.first

        def initialize(**args)
          @args = args
          @args = {} if @args.empty?
        end

        def time_period
          sanitize_time_period(args[TIME_PERIOD_ARG])
        end

        def language
          args[LANGUAGE_ARG]
        end

        def no_args?
          time_period.nil? && language.nil?
        end

        private

        def sanitize_time_period(time_period)
          return nil if time_period.nil?

          unless KNOWN_TIME_PERIODS.include? time_period
            error_msg = "The time period '#{time_period}' is invalid. " \
                        "Supported: #{KNOWN_TIME_PERIODS}"
            raise UnknownTimePeriod.new(error_msg)
          end

          time_period
        end
      end
    end
  end
end
