module Gh
  module Trending
    module Parsers
      # Parses the trending repositories page
      class ReposParser < Parser
        LIST_WITH_REPO_NODES_HTML_CLASS = 'ol.repo-list'.freeze
        DIV_WITH_DESCRIPTION = 'div.py-1'.freeze
        SPAN_WITH_LANGUAGE = 'span.mr-3'.freeze
        SPAN_WITH_STARS = 'span.float-right'.freeze

        def parse(parser_args)
          url = build_request_url_with(parser_args)
          page = agent.get(url)
          parsed_page = page.at(LIST_WITH_REPO_NODES_HTML_CLASS)

          if parsed_page.nil?
            changed_page = :front_page_with_repos
            error = Gh::Trending::Parsers::HtmlChangedError.new(changed_page)
            raise error
          end

          repositories = parsed_page.search('li')
          build_repositories_from_node(repositories)
        end

        def resource_url
          'https://github.com/trending'.freeze
        end

        private

        def build_repositories_from_node(repositories_node)
          repositories_node.map do |repo_node|
            Repository.new extract_repo_data repo_node
          end
        end

        def extract_repo_data(repo_node)
          {
            name:         get_name(repo_node),
            description:  get_description(repo_node),
            language:     get_language(repo_node),
            period_stars: get_stars(repo_node),
            url:          get_repo_url(repo_node)
          }
        end

        def get_name(repo_node)
          repo_node.search('h3')[0].children[1].attributes['href'].value
        end

        def get_description(repo_node)
          repo_node.search(DIV_WITH_DESCRIPTION).text.strip
        end

        def get_language(repo_node)
          repo_node.search(SPAN_WITH_LANGUAGE)[0].text.strip
        end

        def get_stars(repo_node)
          string_with_stars = repo_node.search("div.f6")[0].search("a.muted-link")[0].text.strip
          # Github shows the string "1,263 stars this week"
          # Lets strip only the number.
          string_with_stars.gsub!(',', '')

          digits_array = string_with_stars.match(/\d+/)

          # Since, after replacing the commans with '', there can only possibly
          # be one number in the matched data, so we return the first one
          # and convert it to integer.
          digits_array.to_a[0].to_i
        end

        def get_repo_url(repo_node)
          parsed_url = repo_node.search('h3')
                                .children
                                .search('a')[0]
                                .attributes['href']
                                .value
          build_github_url(parsed_url)
        end
      end
    end
  end
end
